# coding=utf-8

from collections import Counter
from typing import List, Dict
from .datasource import QDDataSource
from .embedding import Embedding
from .const import INTENT_QA
from .common import BotPayload


class IntentBot:
    def __init__(
        self,
        data_source: QDDataSource,
        embedder: Embedding,
        collection: str,
        score: float,
        limit: int,
    ):
        self.data_source = data_source
        self.embdder = embedder
        self.collection = collection
        self.score = float(score)
        self.limit = int(limit)

    def create_collection(self):
        """创建数据集

        Args:
            size (int): 向量大小
            distance (QdrantClient.Distance): 损失函数
        """
        try:
            self.data_source.create_collection(collection=self.collection)
        except Exception as e:
            raise e

    def formatter(self, line: List) -> Dict:
        # line format: 问题，答案(意图)
        # return {"question": line[0], "answer": INTENT_QA}
        return BotPayload(question=line[0], answer=INTENT_QA).to_dict()

    def upload_csv(self, filepath: str):
        """导入csv数据集

        Args:
            filepath: 文件路径
        """
        self.data_source.upload_csv(
            self.collection, self.embdder, filepath, self.formatter
        )

    def complete(self, prompt: str) -> BotPayload:
        points = self.data_source.search(
            prompt,
            self.limit,
            embedder=self.embdder,
            collection=self.collection,
        )
        print(points)

        if points is None or len(points) == 0:
            return None

        payloads = [item.payload for item in points if item.score >= self.score]
        if len(payloads) == 0:
            return None

        answers = [p["answer"] for p in payloads]
        most_common_answer = Counter(answers).most_common(1)[0][0]
        result = list(filter(lambda p: p["answer"] == most_common_answer, payloads))
        return BotPayload.factory(**result[0])
