# coding=utf-8

from typing import List, Dict
from .datasource import QDDataSource
from .embedding import Embedding
from .common import BotPayload


class QABot:
    def __init__(
        self,
        data_source: QDDataSource,
        embedder: Embedding,
        collection: str,
        score: int,
        limit: int,
    ):
        self.data_source = data_source
        self.embdder = embedder
        self.collection = collection
        self.score = float(score)
        self.limit = int(limit)

    def create_collection(self):
        """创建数据集

        Args:
            size (int): 向量大小
            distance (QdrantClient.Distance): 损失函数
        """
        try:
            self.data_source.create_collection(collection=self.collection)
        except Exception as e:
            raise e

    def formatter(self, line: List) -> Dict:
        # line format: 问题，答案，跳转类型
        # return {"question": line[0], "answer": line[1], "type": line[2]}
        # return {"question": line[0], "answer": line[1], "type": "demo"}
        return BotPayload(question=line[0], answer=line[1], type="demo").to_dict()

    def upload_csv(self, filepath: str):
        """导入csv数据集

        Args:
            filepath: 文件路径
        """
        self.data_source.upload_csv(
            self.collection, self.embdder, filepath, self.formatter
        )

    def complete(self, prompt: str, score: int = 0, limit: int = 0) -> BotPayload:
        _limit = self.limit if limit == 0 else limit
        _score = self.score if score == 0 else score

        points = self.data_source.search(
            prompt,
            _limit,
            embedder=self.embdder,
            collection=self.collection,
        )
        if points is None or len(points) == 0:
            return None

        payloads = [item.payload for item in points if item.score >= _score]

        if len(payloads) == 0:
            return None

        return BotPayload.factory(**payloads[0])
